==> locus-0.84.tar.gz: locus-0.84/lib/util/messform.cpp <==
#include "myfndt.h"
#include "messform.h"
#include <string.h>

#define DEBUG_unconspicuous
//#define DEBUG_invasive

CFormatBase::CFormatBase()
{
    reserve(5);
}

CFormatBase &CFormatBase::operator << (const DString &Var)
{
    const char *begin = Var.c_str();
    const char *end = begin + Var.GetLength();
    vector<char> in(begin, end);
    push_back(in);
    return *this;
}

CFormatBase &CFormatBase::operator << (const char *Var)
{
    return operator << (DString(Var));
}

CFormatBase &CFormatBase::operator << (int Var)
{   char Buf[100];

    sprintf(Buf, "%d", Var);
    return operator << (Buf);
}

CFormatBase &CFormatBase::operator << (long Var)
{   char Buf[100];

    sprintf(Buf, "%ld", Var);
    return operator << (Buf);
}

CFormatBase &CFormatBase::operator << (double Var)
{   char Buf[100];

    sprintf(Buf, "%f", Var);
    return operator << (Buf);
}

vector<char> CFormatBase::GetMessageIntern(const DString &Template)
{   int i;

#if defined(DEBUG_invasive)
    printf("enter GetMessageIntern: %s, %d params\n", 
        Template.c_str(),
        size());
#endif

    int FromLen = Template.GetLength();
    CMemPtr<char> From(FromLen + 1);
    memcpy((char *)From, Template.c_str(), FromLen + 1); 

    int VarCount = size();

    int ToSize = FromLen + 1;
    for (i = 0; i < FromLen; ++i)
    {   if (*(((char *)From) + i) == '%')
        {   int c = *(((char *)From) + i + 1);
            if (c >= '0' && c <= '9')
            {   int j = c - '0';
                if (j < VarCount)
                    ToSize += (this->operator[](j)).size();
            }
            ++ToSize;
            ++i;
        }
    }
#if defined(DEBUG_unconspicuous)
    ToSize += 2;
#endif
    CMemPtr<char> To(ToSize);
#if defined(DEBUG_unconspicuous)
    *(((char *)To) + ToSize - 2) = 'B';
    *(((char *)To) + ToSize - 1) = 'C';
#endif
    char *p = (char *)From;
    char *q = (char *)To;
    for (i = 0; i < FromLen; ++i)
    {   if (*(((char *)From) + i) == '%')
        {   int c = *(((char *)From) + i + 1);
		    if (c >= '0' && c <= '9')
		    {   int n = (((char *)From) + i) - p;
                memcpy(q, p, n);
                q += n;
                int j = c - '0';
                if (j < VarCount)
                {   const vector<char> &var = this->operator[](j);
                    const char *b = var.begin();
                    memcpy(q, b, var.size());
                    q += var.size();
                    ++i;
                }
                else
                {   *q = '%'; 
                        // return value of GetMessage is *not* used as a format
                        // string - it is printed as string
                    ++q;
                }
            }
            else
            {   *q = '%'; ++q;
                if (c == '%')
                    ++i;
            }
            p = ((char *)From) + i + 1;
        }
    }
    int n = strlen(p);
    memcpy(q, p, n);
    q += n;
#if defined(DEBUG_unconspicuous)
    Assert(*(((char *)To) + ToSize - 2) == 'B' &&
	*(((char *)To) + ToSize - 1) == 'C');
#endif

#if defined(DEBUG_invasive)
    printf("exit GetMessageIntern:");
    char *x = (char *)To;
    while (x != q)
    {   if (x != (char *)To)
            printf(", ");
        printf("\'%c\'", *x);
        x++;
    }
    printf("\n");
#endif

    return vector<char>((char *)To, q);
}

CFormatRaw::CFormatRaw()
{
}

CFormatRaw &CFormatRaw::operator << (const vector<char> &Var)
{
    push_back(Var);
    return *this;
}

CFormatString::CFormatString()
{
}

DString CFormatString::GetMessage(const DString &Template)
{
    vector<char> Message = GetMessageIntern(Template);
#if defined(DEBUG_unconspicuous)
    vector<char>::iterator i = Message.begin();
    while (i != Message.end())
    {   Assert(*i != 0);
        ++i;
    }
#endif
    Message.push_back(0);
    const char *b = Message.begin();
    return DString(b);
}

char *CFormatString::AllocMessage(const DString &Template)
{
    DString MessStr = GetMessage(Template);
    short Size = MessStr.GetLength() + 1;
    CMemPtr<char> MessArr(Size);
    if (Size > 1)
        memcpy((char *)MessArr, MessStr.c_str(), Size);
    else
        *((char *)MessArr) = 0;
    return MessArr.Drop();
}

==> locus-0.88.tar.gz: locus-0.88/lib/util/messform.cpp <==
#include "myfndt.h"
#include "messform.h"
#include <string.h>

#define DEBUG_unconspicuous
//#define DEBUG_invasive

CFormatBase::CFormatBase()
{
    reserve(5);
}

CFormatBase &CFormatBase::operator << (const DString &Var)
{
    const char *begin = Var.c_str();
    const char *end = begin + Var.GetLength();
    vector<char> in(begin, end);
    push_back(in);
    return *this;
}

CFormatBase &CFormatBase::operator << (const char *Var)
{
    return operator << (DString(Var));
}

CFormatBase &CFormatBase::operator << (int Var)
{   char Buf[100];

    sprintf(Buf, "%d", Var);
    return operator << (Buf);
}

CFormatBase &CFormatBase::operator << (long Var)
{   char Buf[100];

    sprintf(Buf, "%ld", Var);
    return operator << (Buf);
}

CFormatBase &CFormatBase::operator << (double Var)
{   char Buf[100];

    sprintf(Buf, "%g", Var);
    return operator << (Buf);
}

vector<char> CFormatBase::GetMessageIntern(const DString &Template)
{   int i;

#if defined(DEBUG_invasive)
    printf("enter GetMessageIntern: %s, %d params\n", 
        Template.c_str(),
        size());
#endif

    int FromLen = Template.GetLength();
    CMemPtr<char> From(FromLen + 1);
    memcpy((char *)From, Template.c_str(), FromLen + 1); 

    int VarCount = size();

    int ToSize = FromLen + 1;
    for (i = 0; i < FromLen; ++i)
    {   if (*(((char *)From) + i) == '%')
        {   int c = *(((char *)From) + i + 1);
            if (c >= '0' && c <= '9')
            {   int j = c - '0';
                if (j < VarCount)
                    ToSize += (this->operator[](j)).size();
            }
            ++ToSize;
            ++i;
        }
    }
#if defined(DEBUG_unconspicuous)
    ToSize += 2;
#endif
    CMemPtr<char> To(ToSize);
#if defined(DEBUG_unconspicuous)
    *(((char *)To) + ToSize - 2) = 'B';
    *(((char *)To) + ToSize - 1) = 'C';
#endif
    char *p = (char *)From;
    char *q = (char *)To;
    for (i = 0; i < FromLen; ++i)
    {   if (*(((char *)From) + i) == '%')
        {   int c = *(((char *)From) + i + 1);
		    if (c >= '0' && c <= '9')
		    {   int n = (((char *)From) + i) - p;
                memcpy(q, p, n);
                q += n;
                int j = c - '0';
                if (j < VarCount)
                {   const vector<char> &var = this->operator[](j);
                    const char *b = var.begin();
                    memcpy(q, b, var.size());
                    q += var.size();
                    ++i;
                }
                else
                {   *q = '%'; 
                        // return value of GetMessage is *not* used as a format
                        // string - it is printed as string
                    ++q;
                }
            }
            else
            {   *q = '%'; ++q;
                if (c == '%')
                    ++i;
            }
            p = ((char *)From) + i + 1;
        }
    }
    int n = strlen(p);
    memcpy(q, p, n);
    q += n;
#if defined(DEBUG_unconspicuous)
    Assert(*(((char *)To) + ToSize - 2) == 'B' &&
	*(((char *)To) + ToSize - 1) == 'C');
#endif

#if defined(DEBUG_invasive)
    printf("exit GetMessageIntern:");
    char *x = (char *)To;
    while (x != q)
    {   if (x != (char *)To)
            printf(", ");
        printf("\'%c\'", *x);
        x++;
    }
    printf("\n");
#endif

    return vector<char>((char *)To, q);
}

CFormatRaw::CFormatRaw()
{
}

CFormatRaw &CFormatRaw::operator << (const vector<char> &Var)
{
    push_back(Var);
    return *this;
}

CFormatString::CFormatString()
{
}

DString CFormatString::GetMessage(const DString &Template)
{
    vector<char> Message = GetMessageIntern(Template);
#if defined(DEBUG_unconspicuous)
    vector<char>::iterator i = Message.begin();
    while (i != Message.end())
    {   Assert(*i != 0);
        ++i;
    }
#endif
    Message.push_back(0);
    const char *b = Message.begin();
    return DString(b);
}

char *CFormatString::AllocMessage(const DString &Template)
{
    DString MessStr = GetMessage(Template);
    short Size = MessStr.GetLength() + 1;
    CMemPtr<char> MessArr(Size);
    if (Size > 1)
        memcpy((char *)MessArr, MessStr.c_str(), Size);
    else
        *((char *)MessArr) = 0;
    return MessArr.Drop();
}
